#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: extract
#------------------------------------------------------------------------------
# Command to run the main extraction process
#==============================================================================

import argparse, os

import main

import os, sys
LIB_PATH = os.path.dirname(os.path.abspath(__file__)) + '/'
print('Running in ' + LIB_PATH)
os.chdir(LIB_PATH)
sys.path.insert(0, os.path.abspath(LIB_PATH))

# -- Logging
LIB_PATH = os.path.dirname(os.path.abspath(__file__)) + '/'
import logging.config
import logging
LOGGING_CONF_FILE_PATH = f'{LIB_PATH}logging.conf'
logging.config.fileConfig(LOGGING_CONF_FILE_PATH, disable_existing_loggers=False)
logger = logging.getLogger('root')


#==============================================================================
# Default values
#==============================================================================

DEFAULT_SOURCE_TYPE = 'amr' 
DEFAULT_SOURCE_CORPUS = "samples/s1/"


   
#==============================================================================
# Main Process
#==============================================================================

def control_arguments(): 
    arg_parser = argparse.ArgumentParser(
        description=("TENET - Tool for Extraction using Net Extension ",
                     "by (semantic) Transduction"))     
    arg_parser.add_argument("--source_type", nargs='?',
                            default=DEFAULT_SOURCE_TYPE,
                            help="source_type: amr or unl")   
    arg_parser.add_argument("--source_corpus",
                            default=DEFAULT_SOURCE_CORPUS,
                            help="source_corpus: source corpus directory with slash")  
    arg_parser.add_argument("--onto_prefix",
                            default=None,
                            help="onto_prefix: id for the target ontology")   
    arg_parser.add_argument("--out_file_path",
                            default=None,
                            help="out_file_path: path of the output file")  
    arg_parser.add_argument("--technical_dir_path",
                            default=None,
                            help="technical_dir_path: technical directory path")    
    args = arg_parser.parse_args()
    return args
    

if __name__ == '__main__':
    
    args = control_arguments()
    
    if os.path.isfile(args.source_corpus):
        amrld_file_path = args.source_corpus
        main.create_ontology_from_amrld_file(amrld_file_path,
                                             base_ontology_path=None, 
                                             onto_prefix=args.onto_prefix, 
                                             out_file_path=args.out_file_path,
                                             technical_dir_path=args.technical_dir_path)
    
    if os.path.isdir(args.source_corpus):
        amrld_dir_path = args.source_corpus
        main.create_ontology_from_amrld_dir(amrld_dir_path,
                                            base_ontology_path=None, 
                                            onto_prefix=args.onto_prefix, 
                                            out_file_path=args.out_file_path,
                                            technical_dir_path=args.technical_dir_path)