#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Atom Class Net Query Builder
#------------------------------------------------------------------------------
# Class to generate SPARQL query parts related to semantic nets 
#==============================================================================

#==============================================================================
# Importing required modules
#==============================================================================

from .class_net import ClassNet


#==============================================================================
# Net Class
#==============================================================================

class CompositeClassNet(ClassNet):
    """ Class to generate SPARQL query parts related to semantic nets.
    """
    
    #--------------------------------------------------------------------------
    # Constructor(s)
    #--------------------------------------------------------------------------
           
    def __init__(self, num=''):
        
        # -- Net Signature
        self.type_name = 'compositeClass'
        self.type_id = 'Composite_Class_Net'
        self.id = f'?{self.type_name}Net{num}'
        self.type_uri = f'net:{self.type_id}'
        
        # -- Parent init
        super().__init__(signature=True)
        
        # -- Net Attributes
        self.node = f'{self.id}Node'
        self.mother_class_net = f'{self.id}MotherClassNet'
        self.restriction = f'{self.id}Restriction'
        self.restriction01 = f'{self.id}Restriction01'
        
        self.predicate_table.update({
            'node': 'coverNode',
            'mother_class_net': 'hasMotherClassNet',
            'restriction': 'hasRestriction',
            'restriction01': 'hasRestriction01'
            })