#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Individual Net Query Builder
#------------------------------------------------------------------------------
# Class to generate SPARQL query parts related to semantic nets 
#==============================================================================

#==============================================================================
# Importing required modules
#==============================================================================

from .net import Net


#==============================================================================
# Net Class
#==============================================================================

class IndividualNet(Net):
    """ Class to generate SPARQL query parts related to semantic nets.
    """
    
    #--------------------------------------------------------------------------
    # Constructor(s)
    #--------------------------------------------------------------------------
           
    def __init__(self, num=''):
        
        # -- Net Signature
        self.type_name = 'individual'
        self.type_id = 'Individual_Net'
        self.id = f'?{self.type_name}Net{num}'
        self.type_uri = f'net:{self.type_id}'
        
        # -- Parent init
        super().__init__(signature=True)
        
        # -- Net Attributes
        self.base_class_name = f'{self.id}MotherClassNet'
        self.mother_class_net = f'{self.id}MotherClassNet'
        self.individual_label = f'{self.id}IndividualLabel'
        
        self.predicate_table.update({
            'base_class_name': 'hasBaseClassName',
            'mother_class_net': 'hasMotherClassName',
            'individual_label': 'hasIndividualLabel'
            })

        
    #--------------------------------------------------------------------------
    # Method(s) to build 'Construct' parts  
    #--------------------------------------------------------------------------

    def construct(self, **net_attribute):
        query_code = super().construct(**net_attribute)
        return query_code
     
    
    #--------------------------------------------------------------------------
    # Method(s) to build 'Clause' parts  
    #--------------------------------------------------------------------------
    
    # --
    
    
    #--------------------------------------------------------------------------
    # Method(s) to build 'Binding' parts  
    #--------------------------------------------------------------------------
    
    def bind_uri(self, net_name='nameless', node_reference='00'):
        return super().bind_uri(net_name, node_reference)


    
#==============================================================================
# Development Test
#==============================================================================
    
if __name__ == '__main__':
    
    print('\n' + ' *** Development Test ***')    
        
    print('\n' + ' -- test: Atom Class Net')
    net = IndividualNet()
    print(net)
        
    print('\n' + ' -- test: update a test query')
    test_query= f"""[...]
        CONSTRUCT {{
            {net.construct(base_node='?node1', 
                           mother_class_net='?classNet',
                           individual_label='?valueLabel')}
        
        }}
        WHERE {{
            clause_1
            clause_2
            
            {net.complete_clauses_for_construction('?node1')}
            
            {net.bind_uri('{{node1.concept_label}}', 
                          '{{node1.variable_label}}')}
        }}
    """
    print(test_query)
    
    print('\n' + ' *** - ***')