#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Phenomena Net Query Builder
#------------------------------------------------------------------------------
# Class to generate SPARQL query parts related to semantic nets 
#==============================================================================

#==============================================================================
# Importing required modules
#==============================================================================

from .net import Net


#==============================================================================
# Net Class
#==============================================================================

class PhenomenaNet(Net):
    """ Class to generate SPARQL query parts related to semantic nets.
    """
    
    #--------------------------------------------------------------------------
    # Constructor(s)
    #--------------------------------------------------------------------------
           
    def __init__(self, num=''):
        
        # -- Net Signature
        self.type_name = 'phenomena'
        self.type_id = 'Phenomena_Net'
        self.id = f'?{self.type_name}Net{num}'
        self.type_uri = f'net:{self.type_id}'
        
        # -- Parent init
        super().__init__(signature=True)
        
        # -- Net Attributes
        self.phenomena_type = f'{self.id}PhenomenaType'
        self.phenomena_ref = f'{self.id}PhenomenaRef'
        
        self.predicate_table.update({
            'phenomena_type': 'hasPhenomenaType',
            'phenomena_ref': 'hasPhenomenaRef'
            })

    
    #--------------------------------------------------------------------------
    # Method(s) to build 'Construct' parts  
    #--------------------------------------------------------------------------

    def construct(self, **net_attribute):
        query_code = super().construct(**net_attribute)
        return query_code
     
    
    #--------------------------------------------------------------------------
    # Method(s) to build 'Clause' parts  
    #--------------------------------------------------------------------------
    
    def identify_operator(self, num, target_base_id):
        relation_role = f'amr:role_op{num}'
        target_id = f'{target_base_id}{num}'
        return f"""
            # -- Identify operator (amr:op)
            {self.id} {relation_role} {target_id}.""" 
    
    
    def identify_optional_operator(self, num, target_base_id):
        relation_role = f'amr:role_op{num}'
        target_id = f'{target_base_id}{num}'
        return f"""
            # -- Identify optional operator (amr:op)
            OPTIONAL {{ {self.id} {relation_role} {target_id}. }}"""
            
            
    
    #--------------------------------------------------------------------------
    # Method(s) to build 'Binding' parts  
    #--------------------------------------------------------------------------
    
    def bind_uri(self, net_name='nameless', node_reference='00'):
        return super().bind_uri(net_name, node_reference)


    
#==============================================================================
# Development Test
#==============================================================================
    
if __name__ == '__main__':
    
    print('\n' + ' *** Development Test ***')    
        
    print('\n' + ' -- test: Atom Class Net')
    atom_class_net = AtomClassNet()
    print(atom_class_net)
    
        
    print('\n' + ' -- test: construct')
    construct_ctr = atom_class_net.construct(base_node='?node1', 
                                             structure='?structureRef',
                                             class_name='?leaf1ConceptLabel')
    print(construct_ctr)
        
    print('\n' + ' -- test: update a test query')
    test_query= f"""[...]
        CONSTRUCT {{
            {atom_class_net.construct(base_node='?node1', 
                                      structure='?structureRef',
                                      class_name='?leaf1ConceptLabel')}
            
            {atom_class_net.propagate_relations()}
        
        }}
        WHERE {{
            clause_1
            clause_2
            
            {atom_class_net.identify_relations_for_propagation('?node1')}
            
            {atom_class_net.bind_uri('{{node1.concept_label}}',
                                     '{{node1.variable_label}}')}
        }}
    """
    print(test_query)
    
    print('\n' + ' *** - ***')