#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Atom Class Net Query Builder
#------------------------------------------------------------------------------
# Class to generate SPARQL query parts related to semantic nets 
#==============================================================================

#==============================================================================
# Importing required modules
#==============================================================================

from .net import Net


#==============================================================================
# Net Class
#==============================================================================

class RestrictionNet(Net):
    """ Class to generate SPARQL query parts related to semantic nets.
    """
    
    #--------------------------------------------------------------------------
    # Constructor(s)
    #--------------------------------------------------------------------------
           
    def __init__(self, num=''):
        
        # -- Net Signature
        self.type_name = 'restriction'
        self.type_id = 'Restriction_Net'
        self.id = f'?{self.type_name}Net{num}'
        self.type_uri = f'net:{self.type_id}'
        
        # -- Parent init
        super().__init__(signature=True)
        
        # -- Net Attributes
        self.target_node = f'{self.id}TargetNode'
        self.restriction_property = f'{self.id}RestrictionOnProperty'
        self.restriction_net_value = f'{self.id}RestrictionNetValue'
        
        self.predicate_table.update({
            'target_node': 'coverTargetNode',
            'restriction_property': 'hasRestrictionOnProperty',
            'restriction_net_value': 'hasRestrictionNetValue'
            })

    
    #--------------------------------------------------------------------------
    # Method(s) to build 'Construct' parts  
    #--------------------------------------------------------------------------

    def construct(self, **net_attribute):
        query_code = super().define_new_net(**net_attribute)
        return query_code
    
    
    #--------------------------------------------------------------------------
    # Method(s) to build 'Complement Clause' parts  
    #--------------------------------------------------------------------------

    def complete_clauses_for_construction(self):
        query_code = self.identify_structure()
        return query_code
   
    
    #--------------------------------------------------------------------------
    # Method(s) to build 'Binding' parts  
    #--------------------------------------------------------------------------
    
    def bind_uri(self, property_name='nameless', arg_name='nameless'):
        ref1 = f"{self.id}Ref1"
        ref2 = f"{self.id}Ref2"
        ref3 = f"{self.id}Ref3"
        return f"""
            # -- New Restriction Net
            BIND (CONCAT(str(net:), '{self.type_name}') AS {ref1}).
            BIND (CONCAT({ref1}, '_', {property_name}) AS {ref2}).
            BIND (CONCAT({ref2}, '_', {arg_name}) AS {ref3}).
            BIND (uri({ref3}) AS {self.id})."""
            
    
#==============================================================================
# Development Test
#==============================================================================
    
if __name__ == '__main__':
    
    print('\n' + ' *** Development Test ***')    
        
    print('\n' + ' -- test: Net')
    net = AtomClassNet()
    print(net)
        
    print('\n' + ' -- test: update a test query')
    test_query= f"""[...]
        CONSTRUCT {{
            {net.construct(base_node='?node1',
                           class_name='system')}
        
        }}
        WHERE {{
            clause_1
            clause_2
            
            {net.complete_clauses_for_construction('?node1')}
            
            {net.bind_uri('{{node1.concept_label}}',
                          '{{node1.variable_label}}')}
        }}
    """
    print(test_query)
    
    print('\n' + ' *** - ***')