#!/usr/bin/python3.10
# -*- coding: Utf-8 -*-

#==============================================================================
# TENET: Rule to update AMR edge role by adding relation with AMR_Role
#------------------------------------------------------------------------------
# Update AMR edge by adding relation with AMR_Role using comparison between
# label and role ID.
#==============================================================================

from rdflib import Graph

import transduction
from transduction import net
from transduction.query_builder import generate_select_query
from transduction.rdfterm_computer import ( produce_uriref, produce_literal )


#==============================================================================
# Pattern Search: edge{amr:hasAmrRole / role{amr:label}}
#==============================================================================

def __search_pattern(graph):
    select_data_list = ['?edge', '?role']
    clause_list = ['?edge a amr:AMR_Edge.',
                   '?edge amr:hasRoleID ?edgeRoleId.',
                   '?role rdfs:subClassOf* amr:AMR_Role.',
                   '?role amr:label ?roleLabel.',
                   'FILTER ( ?roleLabel != "" ).',
                   'FILTER ( ?edgeRoleId = ?roleLabel ).']
    query_code = generate_select_query(graph, select_data_list, clause_list)
    result_set = graph.query(query_code)
    return query_code, result_set


#==============================================================================
# Construct Method(s)
#==============================================================================

def __update_amr_edge_role(graph, edge, role):
    triple_list = []
    
    # -- Add relation with AMR_Role
    relation = produce_uriref(graph, 'amr:hasAmrRole')
    triple_list.append((edge, relation, role))
    
    return triple_list


#==============================================================================
# Main Method
#==============================================================================

def update_amr_edge_role_1(graph):
    
    # -- Rule Initialization
    rule_label = 'update AMR edge role 1'
    
    # -- Search for patterns
    _, pattern_set = __search_pattern(graph)
    
    # -- Selection Analyzing (1)
    rule_triple_list = []
    for pattern in pattern_set:
        rule_triple_list += __update_amr_edge_role(graph, pattern.edge, pattern.role)
    
    return rule_label, rule_triple_list
