#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Rule to reify AMR element (from AMR-LK to AMR-RDF format)
#------------------------------------------------------------------------------
# Reclassify LK concept to AMR Predicat concept for concept (1) corresponding 
# to a predicat (ns3 concept) and (2) not corresponding to a relation.
#==============================================================================

from rdflib import Graph

import transduction
from transduction import net
from transduction.query_builder import generate_select_query
from transduction.rdfterm_computer import ( produce_uriref, produce_literal )


#==============================================================================
# Pattern Search: leaf{variable, concept}
#==============================================================================
       
def __search_pattern(graph):
    select_data_list = ['?amrlk_concept', '?label_1', '?label_2']
    clause_list = [f'?amrlk_concept a ns3:Concept.',
                   f'FILTER ( CONTAINS(str(?amrlk_concept), str(ns3:)) ).',
                   f'FILTER ( ?amrlk_concept != ns3:Frame ).',
                   f'FILTER ( ?amrlk_concept != ns3:NamedEntity ).',
                   f'FILTER NOT EXISTS {{ ?amrC rdfs:subClassOf amr:AMR_Relation_Concept ; amr:fromAmrLk ?amrlk_concept.}}',
                   f'BIND (strafter(str(?amrlk_concept), str(ns1:)) AS ?label_1).',
                   f'BIND (strafter(str(?amrlk_concept), "#") AS ?label_2).',
                   f'FILTER NOT EXISTS {{ ?amrlk_concept rdfs:comment "bug". }}']
    query_code = generate_select_query(graph, select_data_list, clause_list)
    result_set = graph.query(query_code) 
    return query_code, result_set

    
    
#==============================================================================
# Construct Method(s)
#==============================================================================

def __reclassify_as_amr_predicat_concept(
        graph, amrlk_concept, label_1, label_2):
    
    triple_list = []
    
    label = label_1 if len(label_1) > 0 else label_2
    new_concept_uri =  produce_uriref(graph, f'amr:concept_{label}')
    
    # -- New concept
    relation = produce_uriref(graph, 'rdfs:subClassOf')
    value = produce_uriref(graph, 'amr:AMR_Predicat_Concept')
    triple_list.append((new_concept_uri, relation, value))
    
    # -- Label
    relation = produce_uriref(graph, 'amr:label')
    triple_list.append((new_concept_uri, relation, label))

    # -- AMR-LK trace 
    relation = produce_uriref(graph, 'amr:fromAmrLk')
    triple_list.append((new_concept_uri, relation, amrlk_concept))
    
    # -- Classification of the original concept as LinkedData (for tracing)
    relation = produce_uriref(graph, 'rdfs:subClassOf')
    value = produce_uriref(graph, 'amr:AMR_Linked_Data')
    triple_list.append((amrlk_concept, relation, value)) 
    
    return triple_list

  
    
#==============================================================================
# Main Method
#==============================================================================

def reclassify_concept_2(graph):
    
    # -- Rule Initialization 
    rule_label = 'reclassify AMR-LD concept (2)' 

    # -- Search for patterns 
    _, pattern_set = __search_pattern(graph)
    
    # -- Selection Analyzing (1)
    rule_triple_list = []
    for pattern in pattern_set:
        rule_triple_list += __reclassify_as_amr_predicat_concept(
            graph, pattern.amrlk_concept, pattern.label_1, pattern.label_2)
            
    return rule_label, rule_triple_list