#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Rule to add new variable for reified concept
#------------------------------------------------------------------------------
# Add new variable for reified concept based on the given conditions.
#==============================================================================

from rdflib import Graph, Literal

import transduction
from transduction import net
from transduction.query_builder import generate_select_query
from transduction.rdfterm_computer import ( produce_uriref, produce_literal )


#==============================================================================
# Pattern Search: reifiedConcept
#==============================================================================
       
def __search_pattern(graph):
    select_data_list = ['?originConcept', '?vLetter', '?vLabel']
    clause_list = [f'?reifiedConcept rdfs:subClassOf amr:AMR_Predicat_Concept .',
                   f'?reifiedConcept amr:isReifiedConcept true .',
                   f'?reifiedConcept amr:label ?cLabel .',
                   f'?reifiedConcept amr:fromAmrLk ?originConcept.',
                   f'BIND (strafter(str(?originConcept), str(ns1:)) AS ?rLabel1) .',
                   f'BIND (strafter(str(?originConcept), "#") AS ?rLabel2) .',
                   f'BIND (IF (STRLEN(?rLabel1) > 0, ?rLabel1, ?rLabel2) AS ?rLabel)',
                   f'BIND (SUBSTR(?rLabel, 1, 1) AS ?vLetter)',
                   f'BIND (CONCAT(?vLetter, "9") AS ?vLabel).']
    query_code = generate_select_query(graph, select_data_list, clause_list)
    result_set = graph.query(query_code) 
    return query_code, result_set

    
    
#==============================================================================
# Construct Method(s)
#==============================================================================

def __add_new_variable_for_reified_concept(graph, originConcept, vLetter, vLabel):
    
    triple_list = []
    
    new_variable_uri = produce_uriref(graph, f'amr:variable_{vLabel}')
    
    # -- New variable
    relation = produce_uriref(graph, 'rdf:type')
    value = produce_uriref(graph, 'amr:AMR_Variable')
    triple_list.append((new_variable_uri, relation, value))
    
    # -- Reification flag
    relation = produce_uriref(graph, 'amr:isReifiedVariable')
    value = Literal(True)
    triple_list.append((new_variable_uri, relation, value))
    
    # -- Label
    relation = produce_uriref(graph, 'amr:label')
    triple_list.append((new_variable_uri, relation, vLabel))
    
    # -- Linking new variable to LK role
    relation = produce_uriref(graph, 'rdf:type')
    triple_list.append((new_variable_uri, relation, originConcept))
    
    return triple_list

  
    
#==============================================================================
# Main Method
#==============================================================================

def add_new_variable_for_reified_concept(graph):
    
    # -- Rule Initialization 
    rule_label = 'add new variable for reified concept'

    # -- Search for patterns 
    _, pattern_set = __search_pattern(graph)
    
    # -- Selection Analyzing
    rule_triple_list = []
    for pattern in pattern_set:
        rule_triple_list += __add_new_variable_for_reified_concept(
            graph, pattern.originConcept, pattern.vLetter, pattern.vLabel)
            
    return rule_label, rule_triple_list
 
