#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Rule to extract composite classes (rule 1)
#------------------------------------------------------------------------------
# Net Expansion AMR rule to extract composite classes
# Rule: property(arg0:class, arg1:class) => compositeClass
#==============================================================================

from rdflib import Graph

import transduction
from transduction import net
from transduction.query_builder import generate_select_query
from transduction.naming_computer import define_composite_naming_1, define_restriction_naming


#==============================================================================
# Pattern Search: modality(arg1/arg2:atomProperty) => actionProperty
#==============================================================================

def __search_pattern(graph):
    query_code = ''
    result_set = []
    for arg_relation in ['amr:role_ARG1', 'amr:role_ARG2']:
        select_data_list = ['?property_net']
        clause_list = [f'?phenomena_net a [rdfs:subClassOf* net:Modality_Phenomena_Net].',
                       f'FILTER NOT EXISTS {{ ?phenomena_net a net:Deprecated_Net. }}',
                       f'?property_net a [rdfs:subClassOf* net:Atom_Property_Net].',
                       f'?phenomena_net {arg_relation} ?property_net.']
        query_code = generate_select_query(graph, select_data_list, clause_list)
        result_set += graph.query(query_code)
    return query_code, result_set



#==============================================================================
# Useful Computation Method(s)
#============================================================================== 

def __propagate_relation(target_net, base_net):
    target_net.input_relation_list = base_net.input_relation_list
    target_net.output_relation_list = base_net.output_relation_list
    
    
    
#==============================================================================
# Construct Method(s)
#==============================================================================   

def __construct_action_property_net(graph, atom_property_net):

    # -- Net Composition
    action_property_net = net.ActionPropertyNet(graph)
    action_property_net.compose(atom_property_net)
    
    # -- Data Computation 
    action_property_net.property_type = atom_property_net.property_type
    action_property_net.core_role = atom_property_net.core_role
    action_property_net.target_argument_node = atom_property_net.target_argument_node
    action_property_net.naming = atom_property_net.naming
    action_property_net.property_name = atom_property_net.property_name
    action_property_net.property_name01 = atom_property_net.property_name01
    action_property_net.property_name10 = atom_property_net.property_name10
    action_property_net.property_name12 = atom_property_net.property_name12
    
    # -- Relation Propagation
    __propagate_relation(action_property_net, atom_property_net) 
    
    # -- Finalization
    action_property_net.finalize()
    triple_definition = action_property_net.generate_triple_definition()
    
    return action_property_net, triple_definition

  
    
#==============================================================================
# Main Method
#==============================================================================

def extract_action_property_1(graph):
    
    # -- Rule Initialization 
    rule_label = 'extract action properties (1)' 
    rule_triple_list = []

    # -- Search for patterns 
    _, pattern_set = __search_pattern(graph)
    
    # -- Pattern Analysis
    for pattern in pattern_set:
        atom_property_net = net.AtomPropertyNet(graph, uri=pattern.property_net)
                       
        # -- New Net Construction
        _, triple_list = __construct_action_property_net(graph, atom_property_net)
        rule_triple_list += triple_list
        
        # -- Deprecation: Origin Property Net
        rule_triple_list += atom_property_net.deprecate()
    
    return rule_label, rule_triple_list