#!/usr/bin/python3.10
# -*- coding: Utf-8 -*-

#==============================================================================
# TENET: Rule to fix AMR bug
#------------------------------------------------------------------------------
# Fix AMR bug about planet named system solar
#==============================================================================

from rdflib import Graph, Literal

import transduction
from transduction import net
from transduction.query_builder import generate_select_query
from transduction.rdfterm_computer import ( produce_uriref, produce_literal )


#==============================================================================
# Pattern Search: bug about planet named system solar
#==============================================================================

def __search_pattern(graph):
    select_data_list = ['?originNode', '?originConcept']
    clause_list = ['?originConcept a ns3:Concept.',
                   'FILTER ( CONTAINS(str(?originConcept), str(ns2:intercontinental)) ).',
                    '?originNode a ?originConcept.',
                    '?cloudNode ns2:mod ?originNode.', 
                    '?cloudNode a ns2:cloud.'
                   ]
    query_code = generate_select_query(graph, select_data_list, clause_list)
    result_set = graph.query(query_code)
    return query_code, result_set
            

#==============================================================================
# Construct Method(s)
#==============================================================================

def __add_new_concept(graph, originNode, originConcept, new_label='interstellar'):
    triple_list = []
    
    new_concept_uri = produce_uriref(graph, f'ns2:{new_label}')
    
    # -- New concept
    relation = produce_uriref(graph, 'rdf:type')
    value = produce_uriref(graph, 'ns3:Concept')
    triple_list.append((new_concept_uri, relation, value))
    
    # # -- Label
    # relation = produce_uriref(graph, 'rdfs:label')
    # value = Literal(new_label)
    # triple_list.append((new_concept_uri, relation, value))
    
    # -- Linking between individual and new concept
    relation = produce_uriref(graph, 'rdf:type')
    triple_list.append((originNode, relation, new_concept_uri))
    
    # -- Classification of the original concept as LinkedData (for tracing)
    relation = produce_uriref(graph, 'rdfs:subClassOf')
    value = produce_uriref(graph, 'amr:AMR_Linked_Data')
    triple_list.append((originConcept, relation, value))
    
    # -- Comment the origin concept as 'bug'
    relation = produce_uriref(graph, 'rdfs:comment')
    value = Literal('bug')
    triple_list.append((originConcept, relation, value))
    
    return triple_list


#==============================================================================
# Main Method
#==============================================================================

def fix_amr_bug_2(graph):
    
    # -- Rule Initialization
    rule_label = 'fix AMR bug (2)'
    
    # -- Search for patterns
    _, pattern_set = __search_pattern(graph)
    
    # -- Selection Analyzing (1)
    rule_triple_list = []
    for pattern in pattern_set:  
        rule_triple_list += __add_new_concept(graph, pattern.originNode, pattern.originConcept)
    
    return rule_label, rule_triple_list
