#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Rule to entity classification (2)
#------------------------------------------------------------------------------
# AMR rule to classify classes as entities from part relation
#==============================================================================

from rdflib import Graph

import transduction
from transduction import net
from transduction.query_builder import generate_select_query
from transduction.rdfterm_computer import produce_uriref, produce_literal

ENTITY_CLASS_TYPE = 'base-out:Entity'


#==============================================================================
# Pattern Search: modality_phenomena
#==============================================================================

def __search_pattern(graph):
    query_code = ''
    result_set = []
    for arg_relation in ['amr:role_part']:
        select_data_list = ['?class_net_1', '?class_net_2']
        clause_list = [f'?class_net_1 a [rdfs:subClassOf* net:Class_Net].',
                       f'?class_net_2 a [rdfs:subClassOf* net:Class_Net].',
                       f'?class_net_1 {arg_relation} ?class_net_2.']
        query_code = generate_select_query(graph, select_data_list, clause_list)
        result_set += graph.query(query_code)
    return query_code, result_set


#==============================================================================
# Useful Computation Method(s)
#==============================================================================  

# None
    
    
#==============================================================================
# Construct Method(s)
#==============================================================================

def __classify_net_as_entity(graph, net_uri):
    class_net = net.ClassNet(graph, uri=net_uri)
    class_net.class_type = ENTITY_CLASS_TYPE    
    return class_net.generate_triple_definition()
  
    
#==============================================================================
# Main Method
#==============================================================================

def classify_entity_from_part_relation(graph):
    
    # -- Rule Initialization 
    rule_label = 'classify class net as entity from :part relation' 
    rule_triple_list = []
    
    # -- Search for patterns 
    _, pattern_set = __search_pattern(graph)
    
    # -- Pattern Analysis
    for pattern in pattern_set:
        new_triple_1 = __classify_net_as_entity(graph, pattern.class_net_1)
        new_triple_2 = __classify_net_as_entity(graph, pattern.class_net_2)
        rule_triple_list += (new_triple_1 + new_triple_2)
    
    return rule_label, rule_triple_list