#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Rule to propagate individual by completing Individual Net
#------------------------------------------------------------------------------
# AMR rule to propagate individual based on same base node cover between several classes
#==============================================================================

from rdflib import Graph

import transduction
from transduction import net
from transduction.query_builder import generate_select_query
from transduction.rdfterm_computer import produce_uriref, produce_literal


#==============================================================================
# Pattern Search: individual and net with same base node
#==============================================================================

def __search_pattern_1(graph):
    query_code = ''
    result_set = []
    select_data_list = ['?individualNet', '?classNet']
    clause_list = [
        '?individualNet a net:Individual_Net.',
        '?individualNet net:coverBaseNode ?sameBaseLeaf.',
        '?classNet a [rdfs:subClassOf* net:Class_Net].',
        'FILTER NOT EXISTS {{ ?classNet a net:Deprecated_Net. }}',
        '?classNet net:coverBaseNode ?sameBaseLeaf.'
    ]
    query_code = generate_select_query(graph, select_data_list, clause_list)
    result_set = graph.query(query_code)
    return query_code, result_set


#==============================================================================
# Pattern Search: individual and net with domain link
#==============================================================================

def __search_pattern_2(graph):
    query_code = ''
    result_set = []
    select_data_list = ['?individualNet', '?classNet']
    clause_list = [
        '?classNet a [rdfs:subClassOf* net:Class_Net].',
        '?individualNet a net:Individual_Net.',
        '?classNet amr:role_domain ?individualNet.'
    ]
    query_code = generate_select_query(graph, select_data_list, clause_list)
    result_set = graph.query(query_code)
    return query_code, result_set


#==============================================================================
# Useful Computation Method(s)
#==============================================================================  

# None
    
    
#==============================================================================
# Construct Method(s)
#==============================================================================

def __propagate_individual(graph, individual_net_uri, class_net_uri):
    individual_net = net.IndividualNet(graph, uri=individual_net_uri)
    class_net = net.ClassNet(graph, uri=class_net_uri)
    individual_net.mother_class_net = class_net.uri
    return individual_net.generate_triple_definition()


#==============================================================================
# Main Rule Method: propagate-individual-1
#==============================================================================

def propagate_individual_1(graph):
   
    # -- Rule Initialization 
    rule_label = "Propagate individuals to net with same base node"
    rule_triple_list = []
    
    query_code, pattern_set = __search_pattern_1(graph)
    
    for pattern in pattern_set:
        individual_net_uri = str(pattern[0].toPython())
        class_net_uri = str(pattern[1].toPython())
        
        # Propagate individual by completing Individual Net
        new_triples = __propagate_individual(graph, individual_net_uri, class_net_uri)
        rule_triple_list.extend(new_triples)
    
    return rule_label, rule_triple_list


#==============================================================================
# Main Rule Method: propagate-individual-2
#==============================================================================

def propagate_individual_2(graph):
   
    # -- Rule Initialization 
    rule_label = "Propagate individuals to net with domain link"
    rule_triple_list = []
    
    query_code, pattern_set = __search_pattern_2(graph)
    
    for pattern in pattern_set:
        individual_net_uri = str(pattern[0].toPython())
        class_net_uri = str(pattern[1].toPython())
        
        # Propagate individual by completing Individual Net
        new_triples = __propagate_individual(graph, individual_net_uri, class_net_uri)
        rule_triple_list.extend(new_triples)
    
    return rule_label, rule_triple_list
