#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Rule to modality phenomena classification
#------------------------------------------------------------------------------
# AMR rule to classify modality phenomena
# Rule: modality_phenomena classification
#==============================================================================

from rdflib import Graph

import transduction
from transduction import net
from transduction.query_builder import generate_select_query
from transduction.rdfterm_computer import produce_uriref, produce_literal


#==============================================================================
# Pattern Search: modality_phenomena
#==============================================================================

PHENOMENA_TYPE_RELATION = 'net:hasPhenomenaType' 

MODALITY_TYPE_LIST = ['amr:phenomena_modality_possible',
                      'amr:phenomena_modality_obligation',
                      'amr:phenomena_modality_prohibition']

def __get_query_code(graph, modality_type_uri):
    select_data_list = ['?phenomena_net']
    clause_list = [f'?phenomena_net a [rdfs:subClassOf* net:Phenomena_Net].',
                   f'?phenomena_net {PHENOMENA_TYPE_RELATION} {modality_type_uri}.']
    return generate_select_query(graph, select_data_list, clause_list)


def __search_pattern(graph):
    query_code = ''
    result_set = []
    for modality_type_uri in MODALITY_TYPE_LIST:
        query_code = __get_query_code(graph, modality_type_uri)
        result_set += graph.query(query_code)
    return query_code, result_set



#==============================================================================
# Useful Computation Method(s)
#==============================================================================  

# None    
    
    
#==============================================================================
# Construct Method(s)
#==============================================================================

# None
  
    
#==============================================================================
# Main Method
#==============================================================================

def classify_modality_phenomena(graph):
    
    # -- Rule Initialization 
    rule_label = 'classify modality phenomena' 
    rule_triple_list = []
    
    # -- Search for patterns 
    _, pattern_set = __search_pattern(graph)
    
    # -- Pattern Analysis
    for pattern in pattern_set:
        modality_phenomena_net = net.ModalityPhenomenaNet(graph, uri=pattern.phenomena_net)
        modality_phenomena_net.finalize()
        rule_triple_list += modality_phenomena_net.generate_triple_definition()
    
    return rule_label, rule_triple_list