#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Rule to modality phenomena classification
#------------------------------------------------------------------------------
# AMR rule to classify modality phenomena
# Rule: modality_phenomena classification
#==============================================================================

from rdflib import Graph

import transduction
from transduction import net
from transduction.query_builder import generate_select_query
from transduction.rdfterm_computer import produce_uriref, produce_literal


#==============================================================================
# Pattern Search: modality_phenomena
#==============================================================================

def __search_pattern(graph):
    query_code = ''
    result_set = []
    for arg_relation in ['amr:role_ARG0', 'amr:role_ARG1', 'amr:role_ARG2']:
        select_data_list = ['?property_net']
        clause_list = [f'?left_property_net a [rdfs:subClassOf* net:Property_Net].',
                       f'?property_net a [rdfs:subClassOf* net:Atom_Property_Net].',
                       f'?left_property_net {arg_relation} ?property_net.']
        query_code = generate_select_query(graph, select_data_list, clause_list)
        result_set += graph.query(query_code)
    return query_code, result_set



#==============================================================================
# Useful Computation Method(s)
#==============================================================================  

def __propagate_relation(target_net, base_net):
    target_net.input_relation_list = base_net.input_relation_list
    target_net.output_relation_list = base_net.output_relation_list 
    
    
#==============================================================================
# Construct Method(s)
#==============================================================================

def __construct_atom_class_net(graph, property_net):

    # -- Net Composition
    atom_class_net = net.AtomClassNet(graph)
    atom_class_net.compose(property_net)
    
    # -- Data Computation
    atom_class_net.class_name = property_net.naming

    # -- Net Naming
    atom_class_net.naming = property_net.naming
    
    # -- Relation Propagation
    __propagate_relation(atom_class_net, property_net) 
    
    # -- Finalization
    atom_class_net.finalize()
    triple_definition = atom_class_net.generate_triple_definition()
    
    return atom_class_net, triple_definition
  
    
#==============================================================================
# Main Method
#==============================================================================

def reclassify_argument_property_to_class(graph):
    
    # -- Rule Initialization 
    rule_label = 'reclassify argument property to class' 
    rule_triple_list = []
    
    # -- Search for patterns 
    _, pattern_set = __search_pattern(graph)
    
    # -- Pattern Analysis
    for pattern in pattern_set:
        atom_property_net = net.AtomPropertyNet(graph, uri=pattern.property_net)
        _, triple_list = __construct_atom_class_net(graph, atom_property_net)
        rule_triple_list += triple_list
        rule_triple_list += atom_property_net.deprecate()
    
    return rule_label, rule_triple_list