#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: UNL CTR at 'Data Preprocessing' level
#------------------------------------------------------------------------------
# Module grouping compositional transduction rules (CTR) for the analysis of 
# UNL structures, at 'Data Preprocessing' level
#==============================================================================

#==============================================================================
# Prefix using in CTR
#==============================================================================

prefix = """
PREFIX owl: <http://www.w3.org/2002/07/owl#>
PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>
PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>
PREFIX xsd: <http://www.w3.org/2001/XMLSchema#>
PREFIX unl: <https://unl.tetras-libre.fr/rdf/schema#>
PREFIX net: <https://tenet.tetras-libre.fr/semantic-net#>
PREFIX cprm: <https://tenet.tetras-libre.fr/config/parameters#>
PREFIX req: <https://tenet.tetras-libre.fr/frame/requirement-ontology#>
PREFIX sys: <https://tenet.tetras-libre.fr/system-ontology/>
PREFIX fprm: <https://tenet.tetras-libre.fr/frame/parameters#>
    
"""


#==============================================================================
# CTR Set (CTR as Sparql querries)
#==============================================================================

rule = {}


# ---------------------------------------------
# Preprocessing: Reification Bypass
# ---------------------------------------------

rule['bypass_reification'] = {
    'label': "NO-LABEL",
    'comment': "Bypass reification (extension of UNL relations) ",
    'order': 1.11,
    'query': """
    CONSTRUCT { 
        ?node1 ?unlRel ?node2. 
    }
    WHERE {
        ?rel unl:has_source ?node1.
        ?rel unl:has_target ?node2.
        ?rel a ?unlRel.
    }
    """
}


rule['link_scope_entry'] = {
    'label': "NO-LABEL",
    'comment': """"Link UNL relation to scope entry
    (by connecting the source node of the scope to the entry node of the scope)
    """,
    'order': 1.12,
    'query': """
    CONSTRUCT { 
        ?node1Occ ?unlRel ?node2Occ. 
    }
    WHERE {
        ?scope rdf:type unl:UNL_Scope.
        ?node1Occ unl:is_source_of ?rel.
        ?rel unl:has_target ?scope.
        ?scope unl:is_scope_of ?scopeRel.
        ?scopeRel unl:has_source ?node2Occ.
        ?node2Occ unl:has_attribute \".@entry\".
        ?rel a ?unlRel.
    }
    """
}
 

# ---------------------------------------------
# Preprocessing: ID Definition
# ---------------------------------------------

rule['define_uw_id'] = {
    'label': "NO-LABEL",
    'comment': "Define an ID for each UW (occurrence)",
    'order': 1.2,
    'query': """
    CONSTRUCT { 
       ?uw1 unl:has_id ?uwId.
    }
    WHERE {
       # UW: type UW-Occurrence and substructure of req sentence
       ?uw1 rdf:type unl:UW_Occurrence.
       ?uw1 unl:is_substructure_of ?req.
       # Label(s) / URI
       ?req unl:has_id ?reqId.
       ?uw1 rdfs:label ?uw1Label.
       BIND (IF (CONTAINS(?uw1Label, '('), strbefore(?uw1Label, '('), str(?uw1Label)) AS ?rawConcept).
       BIND (REPLACE(?rawConcept, \" \", \"-\") AS ?concept1)
       BIND (strafter(str(?uw1), \"---\") AS ?numOcc).
       BIND (concat( ?reqId, '_', ?concept1, ?numOcc) AS ?uwId).
    }
    """
}

