#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Class Net
#------------------------------------------------------------------------------
# Class to handle semantic nets 
#==============================================================================

from transduction.net import Net
from transduction.rdfterm_computer import produce_uriref, produce_literal


#==============================================================================
# Net Class
#==============================================================================

class ActionNet(Net):
    """ Class to handle semantic net.
    """
    
    #--------------------------------------------------------------------------
    # Constructor(s)
    #--------------------------------------------------------------------------
           
    def __init__(self, support_graph, uri=None):

        # -- Parent init
        super().__init__(support_graph, uri)
        
        # -- Net Type
        self.type_name = 'action'
        self.type_id = 'Action_Net'
        self.type_uri = produce_uriref(self.support_graph, f'net:{self.type_id}')
        
        # -- Net Attributes
        self.attr_list += ['action_name', 
                           'target_class_net', 'target_individual_net', 
                           'assignee_class_net', 'assignee_individual_net']
        self._action_name = None
        self._target_class_net = None
        self._target_individual_net = None
        self._assignee_class_net = None
        self._assignee_individual_net = None
  
        
    #--------------------------------------------------------------------------
    # Accessors for Net Attributes
    #--------------------------------------------------------------------------
        
    @property
    def action_name(self):
        if self._action_name is None: 
            self._action_name = self.get_value_list_from_graph('action_name')
        return self._action_name
    
    @action_name.setter
    def action_name(self, new_value):
        self._action_name = self.set_attribute_value_list(new_value, produce_literal)
        
        
    @property
    def target_class_net(self):
        if self._target_class_net is None: 
            self._target_class_net = self.get_value_list_from_graph('target_class_net')
        return self._target_class_net
    
    @target_class_net.setter
    def target_class_net(self, new_value):
        self._target_class_net = self.set_attribute_value_list(new_value, produce_uriref)
        
        
    @property
    def target_individual_net(self):
        if self._target_individual_net is None: 
            self._target_individual_net = self.get_value_list_from_graph('target_individual_net')
        return self._target_individual_net
    
    @target_individual_net.setter
    def target_individual_net(self, new_value):
        self._target_individual_net = self.set_attribute_value_list(new_value, produce_uriref)
        
        
    @property
    def assignee_class_net(self):
        if self._assignee_class_net is None: 
            self._assignee_class_net = self.get_value_list_from_graph('assignee_class_net')
        return self._assignee_class_net
    
    @assignee_class_net.setter
    def assignee_class_net(self, new_value):
        self._assignee_class_net = self.set_attribute_value_list(new_value, produce_uriref)
        
        
    @property
    def assignee_individual_net(self):
        if self._assignee_individual_net is None: 
            self._assignee_individual_net = self.get_value_list_from_graph('assignee_individual_net')
        return self._assignee_individual_net
    
    @assignee_individual_net.setter
    def assignee_individual_net(self, new_value):
        self._assignee_individual_net = self.set_attribute_value_list(new_value, produce_uriref)
