#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Axiom Net
#------------------------------------------------------------------------------
# Class to handle semantic nets 
#==============================================================================

from transduction.net import Net
from transduction.rdfterm_computer import produce_uriref, produce_literal


#==============================================================================
# Net Class
#==============================================================================

class AxiomNet(Net):
    """ Class to handle semantic net.
    """
    
    #--------------------------------------------------------------------------
    # Constructor(s)
    #--------------------------------------------------------------------------
           
    def __init__(self, support_graph, uri=None):

        # -- Parent init
        super().__init__(support_graph, uri)
        
        # -- Net Type
        self.type_name = 'axiom'
        self.type_id = 'Axiom_Net'
        self.type_uri = produce_uriref(self.support_graph, f'net:{self.type_id}')
        
        # -- Net Attributes
        self.attr_list += ['axiom_name', 'axiom_uri', 'axiom_net_argument']
        self._axiom_name = None
        self._axiom_uri = None
        self._axiom_net_argument = []
        
        
    #--------------------------------------------------------------------------
    # Accessors for Net Attributes
    #--------------------------------------------------------------------------
        
    @property
    def axiom_name(self):
        if self._axiom_name is None: 
            self._axiom_name = self.get_value_list_from_graph('axiom_name')
        return self._axiom_name
    
    @axiom_name.setter
    def axiom_name(self, new_value):
        self._axiom_name = self.set_attribute_value_list(new_value, produce_literal)
        
        
    @property
    def axiom_uri(self):
        if self._axiom_uri is None: 
            self._axiom_uri = self.get_value_list_from_graph('axiom_uri')
        return self._axiom_uri
    
    @axiom_uri.setter
    def axiom_uri(self, new_value):
        self._axiom_uri = self.set_attribute_value_list(new_value, produce_uriref)
        
        
    @property
    def axiom_net_argument(self):
        if self._axiom_net_argument is None: 
            self._axiom_net_argument = self.get_value_list_from_graph('axiom_net_argument')
        return self._axiom_net_argument
    
    @axiom_net_argument.setter
    def axiom_net_argument(self, new_value):
        self._axiom_net_argument = self.set_attribute_value_list(new_value, produce_uriref)
