#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Class Net
#------------------------------------------------------------------------------
# Class to handle semantic nets 
#==============================================================================

from transduction.net import Net
from transduction.rdfterm_computer import produce_uriref, produce_literal


#==============================================================================
# Net Class
#==============================================================================

class ClassNet(Net):
    """ Class to handle semantic net.
    """
    
    #--------------------------------------------------------------------------
    # Constructor(s)
    #--------------------------------------------------------------------------
           
    def __init__(self, support_graph, uri=None):

        # -- Parent init
        super().__init__(support_graph, uri)
        
        # -- Net Type
        self.type_name = 'class'
        self.type_id = 'Class_Net'
        self.type_uri = produce_uriref(self.support_graph, f'net:{self.type_id}')
        
        # -- Net Attributes
        self.attr_list += ['class_name', 'class_uri', 'class_type', 
                           'mother_class_net', 'root_class_net',
                           'mother_class_net_to_refine',
                           'restriction']
        self._class_name = None
        self._class_uri = None
        self._class_type = None
        self._mother_class_net = None
        self._root_class_net = None
        self._mother_class_net_to_refine = None
        self._restriction = []
  
        
    #--------------------------------------------------------------------------
    # Accessors for Net Attributes
    #--------------------------------------------------------------------------
        
    @property
    def class_name(self):
        if self._class_name is None: 
            self._class_name = self.get_value_list_from_graph('class_name')
        return self._class_name
    
    @class_name.setter
    def class_name(self, new_value):
        self._class_name = self.set_attribute_value_list(new_value, produce_literal)
        
        
    @property
    def class_uri(self):
        if self._class_uri is None: 
            self._class_uri = self.get_value_list_from_graph('class_uri')
        return self._class_uri
    
    @class_uri.setter
    def class_uri(self, new_value):
        self._class_uri = self.set_attribute_value_list(new_value, produce_uriref)
        
        
    @property
    def class_type(self):
        if self._class_type is None: 
            self._class_type = self.get_value_list_from_graph('class_type')
        return self._class_type
    
    @class_type.setter
    def class_type(self, new_value):
        self._class_type = self.set_attribute_value_list(new_value, produce_uriref)

    @property
    def mother_class_net(self):
        if self._mother_class_net is None: 
            self._mother_class_net = self.get_value_list_from_graph('mother_class_net')
        return self._mother_class_net
    
    @mother_class_net.setter
    def mother_class_net(self, new_value):
        self._mother_class_net = self.set_attribute_value_list(new_value, produce_uriref)

    @property
    def root_class_net(self):
        if self._root_class_net is None: 
            self._root_class_net = self.get_value_list_from_graph('root_class_net')
        return self._root_class_net
    
    @root_class_net.setter
    def root_class_net(self, new_value):
        self._root_class_net = self.set_attribute_value_list(new_value, produce_uriref)
            
    @property
    def mother_class_net_to_refine(self):
        if self._mother_class_net_to_refine is None: 
            self._mother_class_net_to_refine = self.get_value_list_from_graph('mother_class_net_to_refine')
        return self._mother_class_net_to_refine
    
    @mother_class_net_to_refine.setter
    def mother_class_net_to_refine(self, new_value):
        self._mother_class_net_to_refine = self.set_attribute_value_list(new_value, produce_uriref)

        
    @property
    def restriction(self):
        if self._restriction == []: 
            self._restriction = self.get_value_list_from_graph('restriction')
        return self._restriction
    
    @restriction.setter
    def restriction(self, new_value):
        self._restriction = self.set_attribute_value_list(new_value, produce_uriref)