#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Composite Property Net
#------------------------------------------------------------------------------
# Class to handle semantic nets 
#==============================================================================

from transduction.net import PropertyNet
from transduction.rdfterm_computer import produce_uriref, produce_literal


#==============================================================================
# Net Class
#==============================================================================

class CompositePropertyNet(PropertyNet):
    """ Class to handle semantic net.
    """
    
    #--------------------------------------------------------------------------
    # Constructor(s)
    #--------------------------------------------------------------------------
           
    def __init__(self, support_graph, uri=None):

        # -- Parent init
        super().__init__(support_graph, uri)
        
        # -- Net Type
        self.type_name = 'compositeProperty'
        self.type_id = 'Composite_Property_Net'
        self.type_uri = produce_uriref(self.support_graph, f'net:{self.type_id}')
        
        # -- Net Attributes
        self.attr_list += ['mother_property_net', 'core_role', 'target_argument_node', 'property_type', 'restriction']
        self._mother_property_net = None
        self._core_role = None
        self._target_argument_node = []
        self._property_type = None
        self._restriction = []

        
    #--------------------------------------------------------------------------
    # Accessors for Net Attributes
    #--------------------------------------------------------------------------
    
    @property
    def mother_property_net(self):
        if self._mother_property_net == None: 
            self._mother_property_net = self.get_value_list_from_graph('_mother_property_net')
        return self._mother_property_net
    
    @mother_property_net.setter
    def mother_property_net(self, new_value):
        self._mother_property_net = self.set_attribute_value_list(new_value, produce_uriref)
      
        
    @property
    def core_role(self):
        if self._core_role is None: 
            self._core_role = self.get_value_list_from_graph('_core_role')
        return self._core_role
    
    @core_role.setter
    def core_role(self, new_value):
        self._core_role = self.set_attribute_value_list(new_value, produce_literal)


    @property
    def target_argument_node(self):
        if self._target_argument_node == []: 
            self._target_argument_node = self.get_value_list_from_graph('_target_argument_node')
        return self._target_argument_node
    
    @target_argument_node.setter
    def target_argument_node(self, new_value):
        self._target_argument_node = self.set_attribute_value_list(new_value, produce_uriref)

        
    @property
    def property_type(self):
        if self._property_type is None: 
            self._property_type = self.get_value_list_from_graph('_property_type')
        return self._property_type
    
    @property_type.setter
    def property_type(self, new_value):
        self._property_type = self.set_attribute_value_list(new_value, produce_uriref)

        
    @property
    def restriction(self):
        if self._restriction == []: 
            self._restriction = self.get_value_list_from_graph('_restriction')
        return self._restriction
    
    @restriction.setter
    def restriction(self, new_value):
        self._restriction = self.set_attribute_value_list(new_value, produce_uriref)
