#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Phenomena Net
#------------------------------------------------------------------------------
# Class to handle semantic nets 
#==============================================================================

from transduction.net import Net
from transduction.rdfterm_computer import produce_uriref, produce_literal


#==============================================================================
# Net Class
#==============================================================================

class PhenomenaNet(Net):
    """ Class to handle semantic net.
    """
    
    #--------------------------------------------------------------------------
    # Constructor(s)
    #--------------------------------------------------------------------------
           
    def __init__(self, support_graph, uri=None):

        # -- Parent init
        super().__init__(support_graph, uri)
        
        # -- Net Type
        self.type_name = 'phenomena'
        self.type_id = 'Phenomena_Net'
        self.type_uri = produce_uriref(self.support_graph, f'net:{self.type_id}')
        
        # -- Net Attributes
        self.attr_list += ['phenomena_type', 'phenomena_ref', 'feature_net']
        self._phenomena_type = None
        self._phenomena_ref = None
        self._feature_net = None
  
        
    #--------------------------------------------------------------------------
    # Accessors for Net Attributes
    #--------------------------------------------------------------------------
        
    @property
    def phenomena_type(self):
        if self._phenomena_type is None: 
            self._phenomena_type = self.get_value_list_from_graph('_phenomena_type')
        return self._phenomena_type
    
    @phenomena_type.setter
    def phenomena_type(self, new_value):
        self._phenomena_type = self.set_attribute_value_list(new_value, produce_uriref)

        
    @property
    def phenomena_ref(self):
        if self._phenomena_ref is None: 
            self._phenomena_ref = self.get_value_list_from_graph('_phenomena_ref')
        return self._phenomena_ref
    
    @phenomena_ref.setter
    def phenomena_ref(self, new_value):
        self._phenomena_ref = self.set_attribute_value_list(new_value, produce_literal)

    @property
    def feature_net(self):
        if self._feature_net is None: 
            self._feature_net = self.get_value_list_from_graph('feature_net')
        return self._feature_net
    
    @feature_net.setter
    def feature_net(self, new_value):
        self._feature_net = self.set_attribute_value_list(new_value, produce_uriref)
