#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Property Net
#------------------------------------------------------------------------------
# Class to handle semantic nets 
#==============================================================================

from transduction.net import Net
from transduction.rdfterm_computer import produce_uriref, produce_literal


#==============================================================================
# Net Class
#==============================================================================

class PropertyNet(Net):
    """ Class to handle semantic net.
    """
    
    #--------------------------------------------------------------------------
    # Constructor(s)
    #--------------------------------------------------------------------------
           
    def __init__(self, support_graph, uri=None):

        # -- Parent init
        super().__init__(support_graph, uri)
        
        # -- Net Type
        self.type_name = 'property'
        self.type_id = 'Property_Net'
        self.type_uri = produce_uriref(self.support_graph, f'net:{self.type_id}')
        
        # -- Net Attributes
        self.attr_list += ['property_name', 'property_uri', 'property_type']
        self._property_name = None
        self._property_uri = None
        self._property_type = None
        
        
    #--------------------------------------------------------------------------
    # Accessors for Net Attributes
    #--------------------------------------------------------------------------
        
    @property
    def property_name(self):
        if self._property_name is None: 
            self._property_name = self.get_value_list_from_graph('property_name')
        return self._property_name
    
    @property_name.setter
    def property_name(self, new_value):
        self._property_name = self.set_attribute_value_list(new_value, produce_literal)
        
        
    @property
    def property_uri(self):
        if self._property_uri is None: 
            self._property_uri = self.get_value_list_from_graph('property_uri')
        return self._property_uri
    
    @property_uri.setter
    def property_uri(self, new_value):
        self._property_uri = self.set_attribute_value_list(new_value, produce_uriref)

    @property
    def property_type(self):
        if self._property_type is None: 
            self._property_type = self.get_value_list_from_graph('_property_type')
        return self._property_type
    
    @property_type.setter
    def property_type(self, new_value):
        self._property_type = self.set_attribute_value_list(new_value, produce_uriref)
