#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Relation Net
#------------------------------------------------------------------------------
# Class to handle semantic nets 
#==============================================================================

from transduction.net import Net
from transduction.rdfterm_computer import produce_uriref, produce_literal


#==============================================================================
# Net Class
#==============================================================================

class RelationNet(Net):
    """ Class to handle semantic net.
    """
    
    #--------------------------------------------------------------------------
    # Constructor(s)
    #--------------------------------------------------------------------------
           
    def __init__(self, support_graph, uri=None):

        # -- Parent init
        super().__init__(support_graph, uri)
        
        # -- Net Type
        self.type_name = 'relation'
        self.type_id = 'Relation_Net'
        self.type_uri = produce_uriref(self.support_graph, f'net:{self.type_id}')
        
        # -- Net Attributes
        self.attr_list += ['subject_net', 'predicate_net', 'object_net']
        self._subject_net = None
        self._predicate_net = None
        self._object_net = None
        
        
    #--------------------------------------------------------------------------
    # Accessors for Net Attributes
    #--------------------------------------------------------------------------
        
    @property
    def subject_net(self):
        if self._subject_net is None: 
            self._subject_net = self.get_value_list_from_graph('subject_net')
        return self._subject_net
    
    @subject_net.setter
    def subject_net(self, new_value):
        self._subject_net = self.set_attribute_value_list(new_value, produce_uriref)
        
    
    @property
    def predicate_net(self):
        if self._predicate_net is None: 
            self._predicate_net = self.get_value_list_from_graph('predicate_net')
        return self._predicate_net
    
    @predicate_net.setter
    def predicate_net(self, new_value):
        self._predicate_net = self.set_attribute_value_list(new_value, produce_uriref)
        
    
    @property
    def object_net(self):
        if self._object_net is None: 
            self._object_net = self.get_value_list_from_graph('object_net')
        return self._object_net
    
    @object_net.setter
    def object_net(self, new_value):
        self._object_net = self.set_attribute_value_list(new_value, produce_uriref)
