#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Restriction Net
#------------------------------------------------------------------------------
# Class to handle semantic nets 
#==============================================================================

from transduction.net import Net
from transduction.rdfterm_computer import produce_uriref, produce_literal


#==============================================================================
# Net Class
#==============================================================================

class RestrictionNet(Net):
    """ Class to handle semantic net.
    """
    
    #--------------------------------------------------------------------------
    # Constructor(s)
    #--------------------------------------------------------------------------
           
    def __init__(self, support_graph, uri=None):

        # -- Parent init
        super().__init__(support_graph, uri)
        
        # -- Net Type
        self.type_name = 'restriction'
        self.type_id = 'Restriction_Net'
        self.type_uri = produce_uriref(self.support_graph, f'net:{self.type_id}')        
        
        # -- Net Attributes
        self.attr_list += ['target_node', 'restriction_property', 'restriction_net_value']
        self._target_node = []
        self._restriction_property = None
        self._restriction_net_value = None
 
        
    #--------------------------------------------------------------------------
    # Accessors for Net Attributes
    #--------------------------------------------------------------------------
        
    @property
    def target_node(self):
        if self._target_node == []: 
            self._target_node = self.get_value_list_from_graph('_target_node')
        return self._target_node
    
    @target_node.setter
    def target_node(self, new_value):
        self._target_node = self.set_attribute_value_list(new_value, produce_uriref)

        
    @property
    def restriction_property(self):
        if self._restriction_property is None: 
            self._restriction_property = self.get_value_list_from_graph('_restriction_property')
        return self._restriction_property
    
    @restriction_property.setter
    def restriction_property(self, new_value):
        self._restriction_property = self.set_attribute_value_list(new_value, produce_uriref)
        
    def has_restriction_property(self):
        return isinstance(self.restriction_property, list) and len(self.restriction_property) > 0

        
    @property
    def restriction_net_value(self):
        if self._restriction_net_value is None: 
            self._restriction_net_value = self.get_value_list_from_graph('_restriction_net_value')
        return self._restriction_net_value
    
    @restriction_net_value.setter
    def restriction_net_value(self, new_value):
        self._restriction_net_value = self.set_attribute_value_list(new_value, produce_uriref)

    def has_restriction_net_value(self):
        return isinstance(self.restriction_net_value, list) and len(self.restriction_net_value) > 0