#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Class Net
#------------------------------------------------------------------------------
# Class to handle semantic nets 
#==============================================================================

from transduction.net import Net
from transduction.rdfterm_computer import produce_uriref, produce_literal


#==============================================================================
# Net Class
#==============================================================================

class RuleNet(Net):
    """ Class to handle semantic net.
    """
    
    #--------------------------------------------------------------------------
    # Constructor(s)
    #--------------------------------------------------------------------------
           
    def __init__(self, support_graph, uri=None):

        # -- Parent init
        super().__init__(support_graph, uri)
        
        # -- Net Type
        self.type_name = 'rule'
        self.type_id = 'Rule_Net'
        self.type_uri = produce_uriref(self.support_graph, f'net:{self.type_id}')
        
        # -- Net Attributes
        self.attr_list += ['rule_relation_name', 'rule_action_net']
        self._rule_relation_name = None
        self._rule_action_net = None
  
        
    #--------------------------------------------------------------------------
    # Accessors for Net Attributes
    #--------------------------------------------------------------------------
        
    @property
    def rule_relation_name(self):
        if self._rule_relation_name is None: 
            self._rule_relation_name = self.get_value_list_from_graph('rule_relation_name')
        return self._rule_relation_name
    
    @rule_relation_name.setter
    def rule_relation_name(self, new_value):
        self._rule_relation_name = self.set_attribute_value_list(new_value, produce_literal)
        
        
    @property
    def rule_action_net(self):
        if self._rule_action_net is None: 
            self._rule_action_net = self.get_value_list_from_graph('rule_action_net')
        return self._rule_action_net
    
    @rule_action_net.setter
    def rule_action_net(self, new_value):
        self._rule_action_net = self.set_attribute_value_list(new_value, produce_uriref)
