#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Value Net
#------------------------------------------------------------------------------
# Class to handle semantic nets 
#==============================================================================

from transduction.net import Net
from transduction.rdfterm_computer import produce_uriref, produce_literal


#==============================================================================
# Net Class
#==============================================================================

class ValueNet(Net):
    """ Class to handle semantic net.
    """
    
    #--------------------------------------------------------------------------
    # Constructor(s)
    #--------------------------------------------------------------------------
           
    def __init__(self, support_graph, uri=None):

        # -- Parent init
        super().__init__(support_graph, uri)
        
        # -- Net Type
        self.type_name = 'value'
        self.type_id = 'Value_Net'
        self.type_uri = produce_uriref(self.support_graph, f'net:{self.type_id}')
        
        # -- Net Attributes
        self.attr_list += ['amr_value', 'value_label']
        self._amr_value = None
        self._value_label = None
  
        
    #--------------------------------------------------------------------------
    # Accessors for Net Attributes
    #--------------------------------------------------------------------------
        
    @property
    def amr_value(self):
        if self._amr_value is None: 
            self._amr_value = self.get_value_list_from_graph('_amr_value')
        return self._amr_value
    
    @amr_value.setter
    def amr_value(self, new_value):
        self._amr_value = self.set_attribute_value_list(new_value, produce_uriref)
        
    @property
    def value_label(self):
        if self._value_label is None: 
            self._value_label = self.get_value_list_from_graph('_value_label')
        return self._value_label
    
    @value_label.setter
    def value_label(self, new_value):
        self._value_label = self.set_attribute_value_list(new_value, produce_literal)
