#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

"""
timedelta_utils.py
------------------------------------------------------------------------------
A utility module for performing date calculations based on specified durations 
(in days, months, or years) relative to the current date. Provides functionalities 
to compute a date in the past or future from the current date using a given time 
unit and quantity.
"""

from datetime import datetime, timedelta

def calculate_date(direction, quantity, unit):
    """
    Calculate a date based on the given direction ("before" or "after"), 
    quantity, and unit ("day", "month", "year").
    
    Parameters:
    - direction (str): Should be "before" or "after".
    - quantity (int): The amount of time.
    - unit (str): Specifies the unit of time ("day", "month", or "year").
    
    Returns:
    - str: The calculated date in 'YYYY-MM-DD' format.
    """
    
    # Get the current date
    now = datetime.now()
    
    # Calculate the timedelta based on the unit
    if unit == "day":
        delta = timedelta(days=quantity)
    elif unit == "month":
        # A simplified approach to handle months
        # This doesn't account for varying days in different months
        delta = timedelta(days=quantity * 30)
    elif unit == "year":
        # A simplified approach to handle years without considering leap years
        delta = timedelta(days=quantity * 365)
    else:
        raise ValueError("The unit should be 'day', 'month', or 'year'.")

    # Apply the direction to get the final date
    if direction == "before":
        new_date = now - delta
    elif direction == "after":
        new_date = now + delta
    else:
        raise ValueError("The direction should be 'before' or 'after'.")

    return new_date.strftime('%Y-%m-%d')
 
