#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: timer
#------------------------------------------------------------------------------
# Module using Python decorators to log method execution time
#==============================================================================

#==============================================================================
# Importing required modules
#==============================================================================

import logging
import time, datetime
from functools import wraps


#==============================================================================
# Parameters
#==============================================================================

# Logging
logger = logging.getLogger(__name__)
   
     
#==============================================================================
# Main Function
#==============================================================================
  
def timed(func):
    """This decorator prints the execution time for the decorated function."""

    @wraps(func)
    def wrapper(*args, **kwargs):
        
        exec_start = time.perf_counter()
        process_start = time.process_time()
        
        result = func(*args, **kwargs)
        
        exec_end = time.perf_counter()
        process_end = time.process_time()
        exec_time = exec_end - exec_start
        exec_time_date = datetime.timedelta(seconds=exec_time)
        process_time = process_end - process_start
        process_time_date = datetime.timedelta(seconds=process_time)
        
        message = "\n" + "  *** Execution Time *** "
        message += "\n" + "----- Function: {0} ({1})".format(func.__name__,
                                                             func.__module__)
        message += "\n" + "----- Total Time: {0}".format(exec_time_date)
        message += "\n" + "----- Process Time: {0}".format(process_time_date)
        message += "\n" + "  *** - *** "
        logger.info(message)
        
        return result

    return wrapper
  
def timer_return(func):
    """This decorator returns the execution time of the decorated function."""

    @wraps(func)
    def wrapper(*args, **kwargs):
        
        exec_start = time.perf_counter()
        
        result = func(*args, **kwargs)
        
        exec_end = time.perf_counter()
        exec_time = exec_end - exec_start
        exec_time_date = datetime.timedelta(seconds=exec_time)
        
        return result, exec_time_date

    return wrapper