#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: OWL Relation Generator Test
#------------------------------------------------------------------------------
# Script to test the OWL relation generator
#==============================================================================

import os
from rdflib import Graph
from rdflib.namespace import RDF

FILE_PATH = f'{os.path.dirname(os.path.abspath(__file__))}'
INPUT_DIR_PATH = f'{FILE_PATH}/test_data/'
OUTPUT_DIR_PATH = f'{FILE_PATH}/test_data/'

TEST_FILE_NAME = 'devGraph-relations-to-generate'
INPUT_GRAPH_PATH = f'{INPUT_DIR_PATH}{TEST_FILE_NAME}.ttl'
OUTPUT_GRAPH_PATH = f'{OUTPUT_DIR_PATH}{TEST_FILE_NAME}.result.ttl'
OUTPUT_GRAPH_URI = f'https://amr.tetras-libre.fr/rdf/{TEST_FILE_NAME}/result'

from context import tenet
from tenet.transduction import rdfterm_computer
from tenet.scheme.amr_master_rule.owl_generation import owl_relation_generator as dev_rule
from tenet.scheme import amr_master_rule as amr_rule


#==============================================================================
# Useful Methods
#==============================================================================

def load_test_graph():
    print(f'\n -- Test Graph Loading')
    graph = Graph()
    graph.parse(INPUT_GRAPH_PATH)
    print(f" ----- Graph Loaded ({len(graph)})")
    return graph

def add_triples_in_graph(graph, triple_list):
    print(f'\n -- Adding triple(s) in graph')       
    print(f" ----- Graph length before update: {len(graph)}")
    print(f" ----- Number of triples to add: {len(triple_list)}")
    for triple in triple_list:
        print(triple)
        graph.add(triple)      
    print(f" ----- Graph length after update: {len(graph)}")
    graph.serialize(destination=OUTPUT_GRAPH_PATH, format='turtle', base=OUTPUT_GRAPH_URI)


#==============================================================================
# Unit Test
#==============================================================================

def unittest_run_rule(graph, rule):    
    print('\n -- Rule Test')
    rule_label, new_triple_list = rule(graph)
    print(f' ----- label: {rule_label}')
    print(f' ----- new_triple_list ({len(new_triple_list)}):')
    add_triples_in_graph(graph, new_triple_list)


#==============================================================================
# Test Script
#==============================================================================

if __name__ == '__main__':
      
    print('\n *** Test Preparation ***')
    graph = load_test_graph()
    
    print('\n *** Unit Test ***')
    unittest_run_rule(graph, dev_rule.generate_owl_relation)
    
    print('\n *** - ***')
