#!/usr/bin/python3.10
# -*- coding: utf-8 -*-

#==============================================================================
# Test Module: Test generate_owl_triple_definition_of_composite_class
#==============================================================================

from rdflib import Graph, URIRef

from context import tenet
from tenet.transduction import net
from tenet.transduction import rdfterm_computer, prefix_handle
from tenet.scheme.amr_master_rule.owl_generation import owl_class_generator as dev_rule
from tenet.scheme import amr_master_rule as amr_rule


def run_test():
    # Create an empty RDF graph
    graph = Graph()

    # Define a test Composite_Class_Net
    class_net = net.CompositeClassNet(graph, uri='https://example.org/class_net_1')
    class_net.class_uri = ['https://example.org/class_1']
    class_net.class_name = ['Class 1']
    class_net.structure = ['Structure 1']

    # Define a logical restriction
    restriction = {
        'onProperty': 'https://example.org/property_1',
        'someValuesFrom': 'https://example.org/some_value'
    }
    class_net.logical_restrictions = [restriction]

    # Call the method to be tested
    triple_list = dev_rule.__generate_owl_triple_definition_of_composite_class(graph, class_net)

    # Display the generated triples
    for triple in triple_list:
        print(triple)

    # Verify the generated triples
    expected_triples = [
        (
            URIRef('https://example.org/class_net_1'),
            URIRef('https://www.w3.org/2000/01/rdf-schema#subClassOf'),
            URIRef('https://example.org/restriction_restriction_0')
        ),
        (
            URIRef('https://example.org/restriction_restriction_0'),
            URIRef('http://www.w3.org/1999/02/22-rdf-syntax-ns#type'),
            URIRef('http://www.w3.org/2002/07/owl#Restriction')
        ),
        (
            URIRef('https://example.org/restriction_restriction_0'),
            URIRef('http://www.w3.org/2002/07/owl#onProperty'),
            URIRef('https://example.org/property_1')
        ),
        (
            URIRef('https://example.org/restriction_restriction_0'),
            URIRef('http://www.w3.org/2002/07/owl#someValuesFrom'),
            URIRef('https://example.org/some_value')
        )
    ]

    assert triple_list == expected_triples


# Run the test
run_test()
 
