#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Extraction Rule Test
#------------------------------------------------------------------------------
# Script to test rules under development
#==============================================================================

import subprocess, os
from rdflib import Graph, Namespace
from rdflib.plugins.sparql import prepareQuery
from rdflib.namespace import NamespaceManager, FOAF, RDF
from rdflib import URIRef, Literal, BNode


FILE_PATH = f'{os.path.dirname(os.path.abspath(__file__))}'
INPUT_DIR_PATH = f'{FILE_PATH}/test_data/'
OUTPUT_DIR_PATH = f'{FILE_PATH}/test_data/'

TEST_FILE_NAME = 'devGraph1'
INPUT_GRAPH_PATH = f'{INPUT_DIR_PATH}{TEST_FILE_NAME}.ttl'
OUTPUT_GRAPH_PATH = f'{OUTPUT_DIR_PATH}{TEST_FILE_NAME}.result.ttl'
OUTPUT_GRAPH_URI = f'https://amr.tetras-libre.fr/rdf/{TEST_FILE_NAME}/result'

from context import tenet

# Module to test
from tenet.transduction.query_builder import (
    generate_select_query,
    generate_ask_query
)

from tenet.transduction.rdfterm_computer import produce_uriref
from tenet.transduction import prefix_handle

INDENT_STR = '            '



#==============================================================================
# Useful Methods
#==============================================================================

def load_test_graph():
    print(f'\n -- Test Graph Loading')
    graph = Graph()
    prefix_handle.update_graph_namespacemanager(graph)
    graph.parse(INPUT_GRAPH_PATH)
    print(f" ----- Graph Loaded ({len(graph)})")
    return graph
    


#==============================================================================
# Test Script
#==============================================================================

# RDF Prefix Definitions
rdf = Namespace("http://www.w3.org/1999/02/22-rdf-syntax-ns#")
rdfs = Namespace("http://www.w3.org/2000/01/rdf-schema#")
net = Namespace("https://tenet.tetras-libre.fr/semantic-net#")

if __name__ == '__main__':
    
    # Load RDF graph
    graph = load_test_graph()
        
    # Test generate_select_query function
    select_data_list = ['var1', 'var2', 'var3']
    clause_list = [(f'?class_net a net:Atom_Class_Net.')]
    query = generate_select_query(graph, select_data_list, clause_list)
    print(f"Generated SELECT query:\n{query}\n")
    
    # Test generate_ask_query function
    query = generate_ask_query(graph, clause_list)
    print(f"Generated ASK query:\n{query}\n")
    
    # # Test is_class_net function
    # net_uri = produce_uriref(graph, 'net:atomClass_sun_s2')
    # query_code, is_class = is_class_net(graph, net_uri)
    # print(f"Specific ASK query:\n{query_code}\n")
    # print(f"Is {net_uri} a net:Class_Net? {is_class}\n")
    
    # # Test is_individual_net function (1)
    # net_uri = produce_uriref(graph, 'net:atomClass_sun_s2')
    # _, is_individual = is_individual_net(graph, net_uri)
    # print(f"Is {net_uri} a subclass of net:Individual_Net? {is_individual}\n")
    
    # # Test is_individual_net function (2)
    # net_uri = produce_uriref(graph, 'net:individual_system_SolarSystem')
    # _, is_individual = is_individual_net(graph, net_uri)
    # print(f"Is {net_uri} a subclass of net:Individual_Net? {is_individual}\n")
