#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Semantic Net Class Test
#------------------------------------------------------------------------------
# Script to test the semantic net classes
#==============================================================================

import subprocess, os

import rdflib
from rdflib import Graph, Namespace, URIRef
from rdflib.namespace import NamespaceManager

FILE_PATH = f'{os.path.dirname(os.path.abspath(__file__))}'
INPUT_DIR_PATH = f'{FILE_PATH}/test_data/'
OUTPUT_DIR_PATH = f'{FILE_PATH}/test_data/'

TEST_FILE_NAME = 'devGraph1'
INPUT_GRAPH_PATH = f'{INPUT_DIR_PATH}{TEST_FILE_NAME}.ttl'
OUTPUT_GRAPH_PATH = f'{OUTPUT_DIR_PATH}{TEST_FILE_NAME}.result.ttl'
OUTPUT_GRAPH_URI = f'https://amr.tetras-libre.fr/rdf/{TEST_FILE_NAME}/result'

from context import tenet

from tenet.transduction.net import Net

from tenet.transduction.net import ClassNet
from tenet.transduction.net import AtomClassNet
from tenet.transduction.net import CompositeClassNet

from tenet.transduction.net import PropertyNet
from tenet.transduction.net import AtomPropertyNet
from tenet.transduction.net import CompositePropertyNet

from tenet.transduction.net import IndividualNet

from tenet.transduction.net import ValueNet

from tenet.transduction.net import PhenomenaNet

from tenet.transduction.net import RestrictionNet

from tenet.transduction.net import AxiomNet

from tenet.transduction.net import RelationNet




#==============================================================================
# Useful Methods
#==============================================================================

def load_test_graph():
    print(f'\n -- Test Graph Loading')
    graph = Graph()
    graph.bind('amr', Namespace('https://amr.tetras-libre.fr/rdf/schema#'))
    graph.bind('net', Namespace('https://tenet.tetras-libre.fr/semantic-net#'))
    graph.bind('base-out', Namespace('https://tenet.tetras-libre.fr/base-ontology#'))
    graph.parse(INPUT_GRAPH_PATH)
    print(f" ----- Graph Loaded ({len(graph)})")
    return graph  
       

def print_net_attributes(net):
    print(f'\n *** Net attributes ({net.type_id})) ***')
    for attr in net.attr_list:
        net_attr_ref = f'net.{attr}'
        print(f' ----- {attr}: {eval(net_attr_ref)}') 
        
        

#==============================================================================
# Development Test
#==============================================================================

def test_set_uri_attribute(graph):
    print('\n -- Set URI attribute')
    uri = 'https://tenet.tetras-libre.fr/semantic-net#atomClass_sun_s2'
    print(f' ----- uri (1): {uri}')
    if isinstance(uri, rdflib.term.URIRef):
        uri = uri.n3(graph.namespace_manager)
    elif isinstance(uri, str):
        uri = URIRef(uri).n3(graph.namespace_manager)
    print(f' ----- uri (1): {uri}')
    
    
def test_get_net_attribute(graph, net_class_str, net_uri):    
    print(f'\n -- {net_class_str}')
    net = eval(f'{net_class_str}(graph, uri=net_uri)')
    print(f' ----- Net URI: {net.uri}')
    print_net_attributes(net)


#==============================================================================
# Test Script
#==============================================================================

if __name__ == '__main__':
      
    print('\n *** Test Preparation ***')
    graph = load_test_graph()
    print('\n \n')
    
    print('\n *** Development Test ***')
    test_set_uri_attribute(graph)
    
    test_get_net_attribute(graph, 'Net', 'https://tenet.tetras-libre.fr/semantic-net#atomClass_sun_s2')

    test_get_net_attribute(graph, 'ClassNet', 'net:atomClass_sun_s2')
    test_get_net_attribute(graph, 'AtomClassNet', 'net:atomClass_sun_s2')
    test_get_net_attribute(graph, 'CompositeClassNet', 'net:compositeClass_orbit-hasManner-conjunction-OR_o2')
    test_get_net_attribute(graph, 'CompositeClassNet', 'net:compositeClass_system-hasPart-sun-and-object_s')

    test_get_net_attribute(graph, 'PropertyNet', 'net:atomProperty_orbit_o2')     
    test_get_net_attribute(graph, 'AtomPropertyNet', 'net:atomProperty_orbit_o2')
    test_get_net_attribute(graph, 'CompositePropertyNet', 'net:compositeProperty_not-direct_d2')
    
    test_get_net_attribute(graph, 'IndividualNet', 'net:individual_system_SolarSystem')  

    test_get_net_attribute(graph, 'ValueNet', 'net:value_SolarSystem_blankNode')  
     
    test_get_net_attribute(graph, 'PhenomenaNet', 'net:phenomena_conjunction-AND_a')
    test_get_net_attribute(graph, 'PhenomenaNet', 'net:phenomena_conjunction-OR_o3') 

    test_get_net_attribute(graph, 'RestrictionNet', 'net:restriction_hasPart_sun')  
    
    test_get_net_attribute(graph, 'AxiomNet', 'net:atomClass_sun_s2')
    
    test_get_net_attribute(graph, 'RelationNet', 'net:relationNet_relation_test_1')
                      
    print('\n \n')