#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Semantic Net Class Test
#------------------------------------------------------------------------------
# Script to test the semantic net classes
#==============================================================================

import subprocess, os
from rdflib import Graph
from rdflib import Namespace
from rdflib.namespace import NamespaceManager

FILE_PATH = f'{os.path.dirname(os.path.abspath(__file__))}'
INPUT_DIR_PATH = f'{FILE_PATH}/test_data/'
OUTPUT_DIR_PATH = f'{FILE_PATH}/test_data/'

TEST_FILE_NAME = 'devGraph1'
INPUT_GRAPH_PATH = f'{INPUT_DIR_PATH}{TEST_FILE_NAME}.ttl'
OUTPUT_GRAPH_PATH = f'{OUTPUT_DIR_PATH}{TEST_FILE_NAME}.result.ttl'
OUTPUT_GRAPH_URI = f'https://amr.tetras-libre.fr/rdf/{TEST_FILE_NAME}/result'

from context import tenet

from tenet.transduction.net import Net

from tenet.transduction.net import ClassNet
from tenet.transduction.net import AtomClassNet
from tenet.transduction.net import CompositeClassNet

from tenet.transduction.net import PropertyNet
from tenet.transduction.net import AtomPropertyNet
from tenet.transduction.net import CompositePropertyNet

from tenet.transduction.net import IndividualNet

from tenet.transduction.net import ValueNet

from tenet.transduction.net import PhenomenaNet

from tenet.transduction.net import RestrictionNet



#==============================================================================
# Utilities
#==============================================================================
      
def print_net_attributes(net):
    print(f'\n *** Net attributes ({net.type_id})) ***')
    for attr in net.attr_list:
        net_attr_ref = f'net.{attr}'
        print(f' ----- {attr}: {eval(net_attr_ref)}') 




#==============================================================================
# Development Test
#==============================================================================

print('\n' + ' *** Development Test ***')
print('\n *** TEST: Semantic Net Composition ***')

print(f'\n -- Test Graph Loading')
graph = Graph()
graph.bind('net', Namespace('https://tenet.tetras-libre.fr/semantic-net#'))
graph.parse(INPUT_GRAPH_PATH)
print(f" ----- Graph Loaded ({len(graph)})")

print('\n -- Nets to compose')
net_1 = AtomClassNet(graph, uri='net:atomClass_object_o')
print(f' ----- net_1: {net_1.uri} ({net_1.naming})') 
net_2 = PropertyNet(graph, uri='net:atomProperty_orbit_o2')
print(f' ----- net_2: {net_2.uri} ({net_2.naming})') 
net_3 = AtomClassNet(graph, uri='net:atomClass_sun_s2')
print(f' ----- net_3: {net_3.uri} ({net_3.naming})') 

print('\n -- New Net constructed by composition')
new_net = CompositeClassNet(graph)
print('\n /// before composition')
print_net_attributes(new_net)

new_net.compose(net_1, net_2, net_3)

print('\n /// after composition')
print_net_attributes(new_net)
    
print('\n' + ' *** - ***')