#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Semantic Net Class Test
#------------------------------------------------------------------------------
# Script to test the semantic net classes
#==============================================================================

import subprocess, os
from rdflib import Graph
from rdflib import Namespace
from rdflib.namespace import NamespaceManager

FILE_PATH = f'{os.path.dirname(os.path.abspath(__file__))}'
INPUT_DIR_PATH = f'{FILE_PATH}/test_data/'
OUTPUT_DIR_PATH = f'{FILE_PATH}/test_data/'

TEST_FILE_NAME = 'devGraph1'
INPUT_GRAPH_PATH = f'{INPUT_DIR_PATH}{TEST_FILE_NAME}.ttl'
OUTPUT_GRAPH_PATH = f'{OUTPUT_DIR_PATH}{TEST_FILE_NAME}.result.ttl'
OUTPUT_GRAPH_URI = f'https://amr.tetras-libre.fr/rdf/{TEST_FILE_NAME}/result'

from context import tenet

from tenet.transduction.rdfterm_computer import __update_uri_with_prefix

from tenet.transduction.net import Net

from tenet.transduction.net import ClassNet
from tenet.transduction.net import AtomClassNet
from tenet.transduction.net import CompositeClassNet

from tenet.transduction.net import PropertyNet
from tenet.transduction.net import AtomPropertyNet
from tenet.transduction.net import CompositePropertyNet

from tenet.transduction.net import IndividualNet

from tenet.transduction.net import ValueNet

from tenet.transduction.net import PhenomenaNet

from tenet.transduction.net import RestrictionNet



#==============================================================================
# Useful Methods
#==============================================================================

def load_test_graph():
    print(f'\n -- Test Graph Loading')
    graph = Graph()
    graph.bind('amr', Namespace('https://amr.tetras-libre.fr/rdf/schema#'))
    graph.bind('net', Namespace('https://tenet.tetras-libre.fr/semantic-net#'))
    graph.bind('base-out', Namespace('https://tenet.tetras-libre.fr/base-ontology#'))
    graph.parse(INPUT_GRAPH_PATH)
    print(f" ----- Graph Loaded ({len(graph)})")
    return graph


def load_test_net(graph, net_uri):
    print(f'\n -- Test Net (uri): {net_uri}')
    net = CompositeClassNet(graph, uri=net_uri)
    net.node = ['https://amr.tetras-libre.fr/rdf/schema#leaf_or_o3']
    net.naming = ' *** test-naming ***'
    attr_ref = ['net.node', 'net.base_node', 'net.structure', 'net.naming']
    for attr in attr_ref:
        print(f' ----- {attr}: {eval(attr)}')    
    return net

    

#==============================================================================
# Unit Test
#==============================================================================       
       
def unittest_generate_triple_definition(graph, net):
    print('\n -- Triple Net Generation')
    
    net.finalize()
    print(f' ----- Net (uri): {net.uri}')
    
    triple_list = net.generate_triple_definition()
    print(f' ----- Triple Number: {len(triple_list)}')
    na_str = 'NA' if len(triple_list) == 0 else ''
    plural_str = 's' if len(triple_list) > 1 else ''
    print(f' ----- Triple{plural_str}: {na_str}')
    n = 0
    for s, p, o in triple_list:
        n += 1
        s = __update_uri_with_prefix(graph, s)
        p = __update_uri_with_prefix(graph, p)
        o = __update_uri_with_prefix(graph, o)
        print(f' [{n}] {s} {p} {o}')
        
    print(f'\n -- Adding triples in test graph')
    print(f" ----- Graph length before update: {len(graph)}")
    for triple in triple_list:
        graph.add(triple)
    print(f" ----- Graph length after update: {len(graph)}")
    
    print(f'\n -- Serialize test graph to {OUTPUT_GRAPH_PATH}')
    graph.serialize(destination=OUTPUT_GRAPH_PATH, format='turtle')
    

#==============================================================================
# Test Script
#==============================================================================

if __name__ == '__main__':
    
    print(f'===========================================================')
    print(f'= UNIT TEST')
    print(f'= module: transduction.net')
    print(f'= method(s): Triple Generation Method')
    print(f'===========================================================')
      
    print('\n *** Test Preparation ***')
    graph = load_test_graph()
    # graph.serialize(destination=OUTPUT_GRAPH_PATH, format='turtle')
    net = load_test_net(graph, 'net:compositeClass_system-hasPart-sun-and-object_s')
    print('\n \n')
    
    # print('\n *** Development Test ***')
    # print('\n \n')
    
    print('\n *** Unit Test ***')
    unittest_generate_triple_definition(graph, net)
                      
    print('\n \n')