#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Semantic Net Class Test
#------------------------------------------------------------------------------
# Script to test the semantic net classes
#==============================================================================

import subprocess, os

import rdflib
from rdflib import Graph, Namespace, URIRef
from rdflib.namespace import NamespaceManager

FILE_PATH = f'{os.path.dirname(os.path.abspath(__file__))}'
INPUT_DIR_PATH = f'{FILE_PATH}/test_data/'
OUTPUT_DIR_PATH = f'{FILE_PATH}/test_data/'

TEST_FILE_NAME = 'devGraph1'
INPUT_GRAPH_PATH = f'{INPUT_DIR_PATH}{TEST_FILE_NAME}.ttl'
OUTPUT_GRAPH_PATH = f'{OUTPUT_DIR_PATH}{TEST_FILE_NAME}.result.ttl'
OUTPUT_GRAPH_URI = f'https://amr.tetras-libre.fr/rdf/{TEST_FILE_NAME}/result'

from context import tenet

from tenet.transduction.net import Net

from tenet.transduction.net import ClassNet
from tenet.transduction.net import AtomClassNet
from tenet.transduction.net import CompositeClassNet

from tenet.transduction.net import PropertyNet
from tenet.transduction.net import AtomPropertyNet
from tenet.transduction.net import CompositePropertyNet

from tenet.transduction.net import IndividualNet

from tenet.transduction.net import ValueNet

from tenet.transduction.net import PhenomenaNet

from tenet.transduction.net import RestrictionNet



#==============================================================================
# Useful Methods
#==============================================================================

def load_test_graph():
    print(f'\n -- Test Graph Loading')
    graph = Graph()
    graph.bind('amr', Namespace('https://amr.tetras-libre.fr/rdf/schema#'))
    graph.bind('net', Namespace('https://tenet.tetras-libre.fr/semantic-net#'))
    graph.bind('base-out', Namespace('https://tenet.tetras-libre.fr/base-ontology#'))
    graph.parse(INPUT_GRAPH_PATH)
    print(f" ----- Graph Loaded ({len(graph)})")
    return graph



def print_net_attributes(net):
    print(f'\n *** Net attributes ({net.type_id})) ***')
    print(f' ----- Net (uri): {net.uri}')
    print(f' ----- Deprecated Net? {net.is_deprecated()}')
    for attr in net.attr_list:
        net_attr_ref = f'net.{attr}'
        print(f' ----- {attr}: {eval(net_attr_ref)}')     
       


#==============================================================================
# Development Test
#==============================================================================

# None

    

#==============================================================================
# Unit Test
#==============================================================================

def unittest_duplicate_as_cloudy_net(graph, net):
    print('\n -- Net Duplication as Cloudy Net')
    new_net = net.duplicate_as_cloudy_net()
    print_net_attributes(new_net)
    return new_net

def unittest_deprecate_net(graph, net):
    print('\n -- Net Deprecation')
    net.deprecate()
    print_net_attributes(net)
    
    

#==============================================================================
# Test Script
#==============================================================================

if __name__ == '__main__':
    
    print(f'===========================================================')
    print(f'= UNIT TEST')
    print(f'= module: transduction.net')
    print(f'= method(s): Net Duplication (as Cloudy Net) Method')
    print(f'=            + Deprecation Method')
    print(f'===========================================================')
      
    print('\n *** Test Preparation ***')
    graph = load_test_graph()
    test_net_1 = AtomClassNet(graph, 'net:atomClass_sun_s2')
    print_net_attributes(test_net_1)
    test_net_2 = CompositeClassNet(graph, 'net:compositeClass_system-hasPart-sun-and-object_s')
    print_net_attributes(test_net_2)
    print('\n \n')
    
    # print('\n *** Development Test ***')
    # print('\n \n')
    
    print('\n *** Unit Test ***')
    new_net_1 = unittest_duplicate_as_cloudy_net(graph, test_net_1)
    new_net_2 = unittest_duplicate_as_cloudy_net(graph, test_net_2)
    unittest_deprecate_net(graph, test_net_1)
    print_net_attributes(new_net_1)
                      
    print('\n \n')