#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Development Extraction Running Script 
#------------------------------------------------------------------------------
# Script to run the extraction process (development)
#==============================================================================

import subprocess, os
from datetime import datetime

FILE_PATH = f'{os.path.dirname(os.path.abspath(__file__))}'
INPUT_DIR_PATH = f'{FILE_PATH}/test_data/'
OUTPUT_DIR_PATH = f'{FILE_PATH}/test_odrl_output/'

from context import tenet


   
#==============================================================================
# AMR Extraction
#==============================================================================

# -----------------------------------------------
# Input / Output Data
# -----------------------------------------------

# -- Input Data
test_data_dir = f'{INPUT_DIR_PATH}amrDocuments/'

uuid_num = '16'
amrld_dir_path = f'{test_data_dir}dev/asail_odrl_sentences/'
amrld_file_path = f'{amrld_dir_path}s{uuid_num}.stog.amr.ttl'
base_output_name = f'aos{uuid_num}'
onto_prefix = f"https://tenet.tetras-libre.fr/demo/clara/{uuid_num}/"


# -- Output references
time_ref = f'{datetime.now().strftime("%Y%m%d")}'
out_dir_path = f'{OUTPUT_DIR_PATH}{base_output_name}-{time_ref}/'
os.makedirs(out_dir_path, exist_ok=True)
out_file_name = f'{base_output_name}_factoid.ttl'
out_file_path = f'{out_dir_path}{out_file_name}'
technical_dir_path = f'{out_dir_path}technical-data/'


# -----------------------------------------------
# Extraction using Main Method(s)
# -----------------------------------------------

# -- Extraction from a file
factoids = tenet.generate_odrl_from_amrld_file(amrld_file_path, 
                                                onto_prefix=onto_prefix,
                                                out_file_path=out_file_path,
                                                technical_dir_path=technical_dir_path)


# -- Extraction from a directory (all files in a directory)
# factoids = tenet.generate_odrl_from_amrld_dir(amrld_dir_path, 
#                                               onto_prefix=onto_prefix,
#                                               out_file_path=out_file_path,
#                                               technical_dir_path=technical_dir_path)

print(factoids)